"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _core = require("@yarnpkg/core");

var _pluginEssentials = require("@yarnpkg/plugin-essentials");

var _typescriptUtils = require("./typescriptUtils");

const getTypesName = descriptor => {
  return descriptor.scope ? `${descriptor.scope}__${descriptor.name}` : `${descriptor.name}`;
};

const afterWorkspaceDependencyAddition = async (workspace, dependencyTarget, descriptor) => {
  if (descriptor.scope === `types`) return;
  const project = workspace.project;
  const configuration = project.configuration;
  const requiresInstallTypes = await (0, _typescriptUtils.hasDefinitelyTyped)(descriptor, configuration);
  if (!requiresInstallTypes) return;
  const cache = await _core.Cache.find(configuration);
  const typesName = getTypesName(descriptor);
  const target = _pluginEssentials.suggestUtils.Target.DEVELOPMENT;
  const modifier = _pluginEssentials.suggestUtils.Modifier.EXACT;
  const strategies = [_pluginEssentials.suggestUtils.Strategy.LATEST];

  const request = _core.structUtils.makeDescriptor(_core.structUtils.makeIdent(`types`, typesName), `unknown`);

  const suggestions = await _pluginEssentials.suggestUtils.getSuggestedDescriptors(request, {
    workspace,
    project,
    cache,
    target,
    modifier,
    strategies
  });
  const nonNullSuggestions = suggestions.filter(suggestion => suggestion.descriptor !== null);
  if (nonNullSuggestions.length === 0) return;
  const selected = nonNullSuggestions[0].descriptor;
  if (selected === null) return;
  workspace.manifest[target].set(selected.identHash, selected);
};

const afterWorkspaceDependencyRemoval = async (workspace, dependencyTarget, descriptor) => {
  if (descriptor.scope === `types`) return;
  const target = _pluginEssentials.suggestUtils.Target.DEVELOPMENT;
  const typesName = getTypesName(descriptor);

  const ident = _core.structUtils.makeIdent(`types`, typesName);

  const current = workspace.manifest[target].get(ident.identHash);
  if (typeof current === `undefined`) return;
  workspace.manifest[target].delete(ident.identHash);
};

const beforeWorkspacePacking = (workspace, rawManifest) => {
  if (rawManifest.publishConfig && rawManifest.publishConfig.typings) rawManifest.typings = rawManifest.publishConfig.typings;

  if (rawManifest.publishConfig && rawManifest.publishConfig.types) {
    rawManifest.types = rawManifest.publishConfig.types;
  }
};

const plugin = {
  hooks: {
    afterWorkspaceDependencyAddition,
    afterWorkspaceDependencyRemoval,
    beforeWorkspacePacking
  }
}; // eslint-disable-next-line arca/no-default-export

var _default = plugin;
exports.default = _default;